<?php
/**
 * YandexMap Class File
 * @author Daneel S. Yaitskov <rtfm.rtfm.rtfm@gmail.com>
 * @link http://daneel0yaitskov.000space.com.
 * @version 0.1
 */

/**
 * YandexMap creates a div for Yandex Map library, load the library
 * and renders map of Earth in the div with it.
 *
 * Required widget's parameter is sitekey.
 *
 */
class YandexMap extends CWidget {
  /**
   * Widght and height of the map div in pixels. By default 600x400.
   * @var integer
   */
  public $width, $height;
  /**
   * There is array of marked points on the map. An item of the array
   * may be array with keys: text, x, y or object with IMapObject interface.
   * @var array
   * @access public
   */
  public $places;
  /**
   * It's  coordinates of the center of a map where $center[0]
   * is latitude and $center[1] is longitude.
   * @var array( x, y )
   * @access public
   */
  public $center;
  /**
   * Site's key (API-key) is base64 string. It is issued by yandex for your site specially.
   * It also may be set via yandex-map-api-key application parameter.
   * @var string 
   * @access public
   */
  public $sitekey;
  /**
   * It's a scale of the map. It has range from 0 ( all earth ) to 17 ( hours, streets ).
   * By default is 14. Default value can be change with yandex-map-scale application parameter.
   * @var int
   * @access public   
   */
  public $scale;
  /**
   * It's map mode. It can be three values: map, sattelite or hybrid.
   * By default it's 'map'. Default value can be changed via yandex-map-mode
   * application parameter.
   * @var string
   * @access public
   */
  public $mode; 
  
  public function init(){
    $params = Yii::app()->params;
    if ( !isset ( $this->width ) ) 
      $this->width = $params->hasProperty ( 'yandex-map-width' ) ?
                                $params->itemAt ( 'yandex-map-width' ) : 600 ;

    if ( !preg_match( '/^[0-9]{1,5}$/', $this->width ) )
       throw new CException ( 'The width parameter is not valid:'
                              . $this->width );
    
    if ( !isset ( $this->height ) ) 
      $this->height = $params->hasProperty ( 'yandex-map-height' ) ?
                                $params->itemAt ( 'yandex-map-height' ) : 400 ;


    if ( !preg_match( '/^[0-9]{1,5}$/', $this->height ) )
       throw new CException ( 'The height parameter is not valid:'
                              . $this->height );
    if ( !isset ( $this->sitekey ) ){
      if ( $params->hasProperty ( 'yandex-map-api-key' ) )
        $this->sitekey = $params->itemAt( 'yandex-map-api-key' );
      else
        throw new CException ( 'The sitekey parameter is not set' );
    }
    
    if ( ! isset ( $this->mode ) )
      $this->mode = $params->hasProperty( 'yandex-map-mode' ) ?
        $params->itemAt( 'yandex-map-mode' ) : 'MAP';
    else {
      $this->mode = strtoupper( $this->mode );
      if ( in_array ( strtoupper( $this->mode ),
                      array ( 'MAP', 'SATTELITE', 'HYBRID' ) ) )
        throw new CException ( 'Invalid value of the \'mode\' parameter: \''
                               . $this->mode . '\'' );
    }

    if ( ! isset ( $this->scale ) )
      $this->scale = $params->hasProperty( 'yandex-map-scale' ) ?
        $params->itemAt( 'yandex-map-scale' ) : 14;
    elseif ( !preg_match( '/^[0-9]{1,2}$/', $this->scale )
           or $this->scale < 0 or $this->scale > 18 )
      throw new CException ( 'Invalid value of the \'scale\' parameter: \''
                             . $this->scale . '\'' );

    if ( isset ( $this->places ) )
      foreach( $this->places as $i => $mp )
        if ( ! ( is_array ( $mp )
                 and isset ( $mp['x'] )
                 and isset ( $mp['y'] )
                 and isset ( $mp['text'] ) )
             and ! ( is_object ( $mp )
                     and $mp instanceof IMapObject ) )
          throw new CException ( "Element $i of places is not an array nor an IMapObject object" );
      
    // if center is not set obviously then set center as coordinate of first object
    if ( ! isset ( $this->center ) && count ($this->places) ){
      if ( is_array ( $this->places[0] ) )
        $this->center = array ( $this->places[0]['x'],
                                $this->places[0]['y']
                                );
      else
        $this->center = array ( $this->places[0]->latitude(),
                                $this->places[0]->longitude()
                                );        
    }elseif ( ! is_array( $this->center )
                or ! isset( $this->center[0] )
                or ! isset( $this->center[1] ) )
            throw new CException ( 'Invalid value of center parameter' );
    
    // map objects

    $cs = Yii::app()->clientScript;
    $cs->registerScriptFile( 'http://api-maps.yandex.ru/1.1/index.xml?key=' . $this->sitekey );
    $id = '_yandex_yii_' . $this->getId();
    $thescript = <<<EOF
      var map$id = new YMaps.Map(document.getElementById("{$this->getId()}"));
      map{$id}.addControl(new YMaps.TypeControl());
      map{$id}.addControl(new YMaps.ToolBar());
      map{$id}.setType(YMaps.MapType.{$this->mode});
      map{$id}.addControl(new YMaps.Zoom());
      
EOF;
    if ( isset( $this->center ) ) {
       $thescript .= <<<EOF
         map{$id}.setCenter(new YMaps.GeoPoint({$this->center[0]},
                                               {$this->center[1]}),
                            {$this->scale});
       
EOF;
    }
    
    foreach( $this->places as $mp ){
      if ( is_array( $mp ) ){
         $x = $mp['x'] ;
         $y = $mp['y'] ;
         $text = $mp['text'];
      }else {
         $x = $mp->latitude(); 
         $y = $mp->longitude();
         $text= $mp->text();
      }
      $thescript .= <<<EOF
        var placemark$id = new YMaps.Placemark( new YMaps.GeoPoint( $x, $y));
        placemark{$id}.setIconContent('$text');
        map{$id}.addOverlay(placemark$id);

EOF;
    }
    $cs->registerScript( 'yandex-map-instance' . $this->getId(),
                         $thescript,
                         CClientScript::POS_LOAD );
    /* this protectes yandex's map ballon from Yii's screen.css */    
    $style = '.yii-yandex-map table { width: auto; margin-bottom: auto; }';
    $cs->registerCss( 'yii-yandex-map' , $style );
  }
  public function run(){
    echo CHtml::tag( 'div',
                     array ( 'id' => $this->getId(),
                             'class' => 'yii-yandex-map',
                             'style' => 'width:'.$this->width.'px;'
                             .'height:' . $this->height . 'px;' ) );
                             
  }
}

?>
